/*
 * PingRequest.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.ping.handlers;

import com.veraxsystems.icmp.jni.IcmpJniBridge;
import com.veraxsystems.icmp.ping.PingResponseHandler;
import com.veraxsystems.icmp.util.TimeUtil;

import java.net.InetAddress;


/**
 * Class contains data for passing ICMP request between application, ICMP {@link Sender} and
 * ICMP {@link Receiver}.
 */
public class PingRequest  implements Comparable<PingRequest> 
{
   private final int messageId;
   private final PingResponseHandler handler;
   private final InetAddress address;
   private final byte[] data;
   private final int dataLength;
   private final long creationTime;
   private final long timeout;
   private long sendTime;

   static private final byte[] defaultData = new byte[IcmpJniBridge.MAX_DATA_LENGTH];
   static private final byte[] defaultDataPattern = { 0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77 };
   
   static 
   {
      for(int dest = 0; dest < defaultData.length; dest += defaultDataPattern.length)
      {
         System.arraycopy(defaultDataPattern, 0, defaultData, dest, Math.min(defaultDataPattern.length, defaultData.length - dest));
      }
   }
   
   /**
    * Constructor initializes all class attributes. Te only attribute that may be changed after 
    * object is created is <code>sendTime</code> (it is changed when object is passed between 
    * <code>Sender</code> and >code>Receiver</code>.
    * @param addressVal address of host echo request is sent to
    * @param dataVal data to be sent (when null default data is used)
    * @param dataLengthVal the length of data to be sent (its value cannot be bigger than data buffer length)
    * @param timeoutVal send timeout in milliseconds
    * @param messageIdVal identifier of ICMP message
    * @param handlerVal result handler object
    */
   public PingRequest(InetAddress addressVal, byte[] dataVal, int dataLengthVal, long timeoutVal, int messageIdVal, PingResponseHandler handlerVal) 
   {
      creationTime = TimeUtil.currentTimeMillis();
      address = addressVal;
      if (dataVal != null)
         data = dataVal;
      else
         data = defaultData;
      dataLength = dataLengthVal;
      timeout = timeoutVal;
      messageId = messageIdVal;
      handler = handlerVal;
      sendTime = 0;
   }

   /**
    * Copy constructor
    * @param request source object to be copied
    */
   public PingRequest(PingRequest request) 
   {
      creationTime = request.getCreationTime();
      address = request.getAddress();
      data = request.getData();
      dataLength = request.getDataLength();
      messageId = request.getMessageId();
      timeout = request.getTimeout();
      handler = request.getHandler();
      sendTime = request.getSendTime();
   }

   /**
    * Get time when object was created.
    * @return creation time in milliseconds
    */
   public long getCreationTime() {
      return creationTime;
   }

   /**
    * Get timeout attribute
    * @return timeout value in milliseconds
    */
   public long getTimeout() {
      return timeout;
   }
   /**
    * Get ICMP request destination address
    * @return network address
    */
   public InetAddress getAddress() 
   {
      return address;
   }

   /**
    * Get ICMP EchoRequest data that shall be send
    * @return data buffer
    */
   public byte[] getData() 
   {
      return data;
   }

   /**
    * Get result handler object that is executed when ICMP result is obtained
    * @return handler object 
    */
   public PingResponseHandler getHandler() 
   {
      return handler;
   }

   /**
    * Get request due time (when request timeout occurs)
    * @return request end time in milliseconds
    */
   public long getEndTime() 
   {
      return creationTime + timeout;
   }
   
   /**
    * Get identifier of message related to this request (see {@link IcmpJniBridge} API 
    * for more information.
    * @return request identifier
    */
   public int getMessageId() 
   {
      return messageId;
   }

   /**
    * Get time the request was handled by sender
    * @return sent time in milliseconds
    */
   public long getSendTime() 
   {
      return sendTime;
   }
   
   /**
    * Set time when request was handled (sent) by <code>Sender</code>
    */
   public void updateSendTime() 
   {
      sendTime = TimeUtil.currentTimeMillis();
      return;
   }
   
   
   /**
    * Compares this object with the specified object for sorting. Compare two objects and 
    * returns a -1, 0, or +1 integer as this object is less than, equal to, or greater than the specified object.
    * @param o object to be compared
    * @return Returns -1, 0, +1 integer as this object is less than, equal to, or greater than the specified object.
    */
   public int compareTo(PingRequest o) 
   {
      return Long.signum(getEndTime() - o.getEndTime());
   }

   /**
    * Get length of data to be sent. 
    * @return data length
    */
   public int getDataLength() 
   {
      return dataLength;
   }
   
}
